/* global Fapi */

/**
 * Objet de configuration de l'accès à une fichothèque
 * L'objet options permet de conserver des options avec l'objet
 * 
 * @constructor
 * @param {string} fichothequeUrl URL de la fichothèque (doit être fini par un /)
 * @param {Object} options Options conservées par la configuration
  */
Fapi.Config = function (fichothequeUrl, options) {
    this.fichothequeUrl = fichothequeUrl;
    this.publicApi = false;
    this.version = "action";
    this.apiName = "";
    this.options =   {};
    this.checkOptions(options);
};

Fapi.Config.prototype.isPublicApi = function () {
    return this.publicApi;
};

/**
 * Retourne l'URL correspondant à une action sur une fichothèque
 * 
 * @param {string} actionName Nom de l'action
 * @returns {string} URL de l'action
 */
Fapi.Config.prototype.getActionUrl = function (actionName) {
    return this.fichothequeUrl + actionName;
};

Fapi.Config.prototype.getApiUrl = function (name, version, endpoint) {
    return this.fichothequeUrl + "api/" + version + "/" + name + "/" + endpoint;
};


/**
 * Retourne l'URL correspondant à une action sur une fichothèque
 * 
 * @param {string} endPointName Nom du point de terminaison
 * @returns {Fapi.Config.EndPoint} endPoint Définition du point de terminaison
 */
Fapi.Config.prototype.getEndPoint = function (endPointName) {
    var config = this;
    switch(this.version) {
        case "action":
            return _getActionEndPoint();
        case "v1":
            return _getV1EndPoint();
        default:
            throw new Exception("Unknown version: " + this.version);
    }
    
    function _getActionEndPoint() {
        switch(endPointName) {
            case "fiche":
                return __newActionEndpoint("corpus", "fiche");
            case "fiche-array":
                return __newActionEndpoint("pioche", "fiche");
             case "motcle":
                return __newActionEndpoint("thesaurus", "motcle");
            case "motcle-array":
                return __newActionEndpoint("pioche", "motcle");
            case "fiches":
                return __newActionEndpoint("corpus", "fiches");
            case "thesaurus":
                return __newActionEndpoint("thesaurus", "thesaurus");
            default :
                throw new Exception("Unknown endpoint: " + endPointName);
        }
        
        function __newActionEndpoint(actionName, jsonName) {
            return config.getActionEndPoint(actionName, jsonName);
        }
        
    }
    
    function _getV1EndPoint() {
        switch(endPointName) {
            case "fiche":
                return __newV1EndPoint("fiche");
            case "fiche-array":
                return __newV1EndPoint("fiche-array");
            case "motcle":
                return __newV1EndPoint("motcle");
            case "motcle-array":
                return __newV1EndPoint("motcle-array");
             case "fiches":
                return __newV1EndPoint("fiches");
            case "thesaurus":
                return __newV1EndPoint("thesaurus");
            default :
                throw new Exception("Unknown endpoint: " + endPointName);
        }
        
        function __newV1EndPoint(name) {
            return new Fapi.Config.EndPoint(config.getApiUrl(config.apiName, config.version, name), "", "v1");
        }
    }
 
};

Fapi.Config.prototype.getActionEndPoint = function (actionName, jsonName) {
    return new Fapi.Config.EndPoint(this.getActionUrl(actionName), jsonName, "action");
};


/**
 * Retourne l'URL de consultation d'une fiche
 * 
 * @param {string} corpus Nom du corpus
 * @param {number} id Identifiant de la fiche
 * @param {string} [format] Nom du gabarit d'affichage à utiliser
 * @returns {string} URL de la fiche
 */
Fapi.Config.prototype.getFicheUrl = function (corpus, id, format) {
    var url = this.fichothequeUrl + "fiches/" + corpus + "-" + id;
    if (format) {
        url += "-" + format;
    }
    url = url + ".html";
    return url;
};

Fapi.Config.prototype.checkOptions = function (source) {
    var config = this;
    if (!source) {
        return;
    }
    for(let key in source) {
        let value = source[key];
        switch(key) {
            case "api":
                config.version = value.version;
                config.publicApi = value.public;
                config.apiName = value.name;
                break;
            case "version":
                config.version = value;
                break;
            case "public":
                config.publicApi = value;
                break;
            case "name":
                config.apiName = value;
                break;
            default:
                config.options[key] = value;
        }
    }
};

Fapi.Config.prototype.ping = function (argObject) {
    Fapi.Ajax.exit(this, argObject);
};

Fapi.Config.prototype.exit = function (argObject) {
    Fapi.Ajax.exit(this, argObject);
};

Fapi.Config.prototype.loadFiche = function (argObject) {
    let corpus = argObject.corpus;
    let id = argObject.id;
    if (argObject.requestParameters) {
        if (corpus) {
            argObject.requestParameters.corpus = corpus;
        }
        if (id) {
            argObject.requestParameters.id = id;
        }
    } else {
        argObject.requestParameters = {
            corpus: corpus,
            id: id
        };
    }
    Fapi.Ajax.loadFiche(this, argObject);
};

Fapi.Config.prototype.loadFicheArray = function (argObject) {
    let ficheQuery = argObject.ficheQuery;
    if (!(ficheQuery instanceof Fql.FicheQuery)) {
        ficheQuery = new Fql.FicheQuery(ficheQuery);
    }
    Fapi.Ajax.loadFicheArray(this, ficheQuery, argObject);
};

Fapi.Config.prototype.loadMotcle = function (argObject) {
    let thesaurus = argObject.thesaurus;
    let id = argObject.id;
    let idalpha = argObject.idalpha;
    if (argObject.requestParameters) {
        if (thesaurus) {
            argObject.requestParameters.thesaurus = thesaurus;
        }
    } else {
        argObject.requestParameters = {
            thesaurus: thesaurus
        };
    }
    if (id) {
        argObject.requestParameters.id = id;
    }
    if (idalpha) {
        argObject.requestParameters.idalpha = idalpha;
    }
    Fapi.Ajax.loadMotcle(this, argObject);
};


Fapi.Config.prototype.loadMotcleArray = function (argObject) {
    let motcleQuery = argObject.motcleQuery;
    if (!(motcleQuery instanceof Fql.MotcleQuery)) {
        motcleQuery = new Fql.MotcleQuery(motcleQuery);
    }
    Fapi.Ajax.loadMotcleArray(this, motcleQuery, argObject);
};

Fapi.Config.prototype.loadThesaurus = function (argObject) {
    let thesaurus = argObject.thesaurus;
    if (argObject.requestParameters) {
        if ((!thesaurus) && (argObject.requestParameters.thesaurus)) {
            thesaurus = argObject.requestParameters.thesaurus;
        }
    }
    Fapi.Ajax.loadThesaurus(this, thesaurus, argObject);
};

Fapi.Config.prototype.loadFiches = function (argObject) {
    let queries = argObject.ficheQueries;
    if ((!queries) && (argObject.ficheQuery)) {
        queries = argObject.ficheQuery;
    }
    Fapi.Ajax.loadFiches(this, queries, argObject);
};

